/////////////////////////////////////////////////////////////////////////
//          Hier befinden sich alle Routinen um die LCD-Anzeige        //
//                             zu steuern                              //
//  lcd_putc(x)  gibt x auf den LCD-Display auf der aktuellen          //
//                            Position aus                             //
//        Folgende Eigenschaften knnen in dieser Funktion             //
//                            angegeben werden:                        //
//                        \c  lscht den Display                       //
//                        \1  Geht zur 1. Zeile                        //
//                        \2  Geht zur 2. Zeile                        //
//                        \3  Geht zur 3. Zeile                        //
//                        \4  Geht zur 4. Zeile                        //
//                                                                     //
//    lcd_gotoxy(x,y) Setzt das LCD-Display auf die X-Y Koordinaten    //
//          Die Koordinaten 1,1 befinden sich oben links im Display    //
//                                                                     //
//    lcd_init()               Initialisiert das LCD-Display           //
//                                                                     //
// Port C ist fr die LCD-Anzeige folgendermaen definiert:            //
//     RB1  RS                                                         //
//     RB2  RW  (wird in diesem Beispiel nicht bentigt)               //
//     RB3  ENABLE                                                     //
//     RB4  D4                                                         //
//     RB5  D5                                                         //
//     RB6  D6                                                         //
//     RB7  D7                                                         //
/////////////////////////////////////////////////////////////////////////

#define lcd_type 2           // 0=5x7, 1=5x10, 2=2 lines



// These bytes need to be sent to the LCD to start it up.
byte CONST LCD_INIT_STRING[4] = {0x20 | (lcd_type << 2), 0xc, 1, 6};
// 0x20 = 100000|1000 ^= 2 Zeilen-Display;   mit lcd_type=2=10(BIN) ???
// 0xc  = 1100 ^= Display an
// 1    = Display lschen
// 6    = 110 = Adresspointer inkrementieren und Displayinhalt nicht schieben


void lcd_send_nibble( byte n ) {
      portb.lcddata = n;
      nop();
      porta.enable = 1;
      delay_us(2);
      porta.enable = 0;
}

void lcd_send_byte( byte address, byte n ) {
      porta.rs = 0;
      delay_ms(2);
      porta.rs = address;
      nop();
      //portb.rw = 0;
      //nop();
      porta.enable = 0;
      lcd_send_nibble(n >> 4);
      lcd_send_nibble(n & 0xf);
}

void lcd_init() {
    byte i;

    porta.rs = 0;               // siehe 4-Bit Initialisierung (LCD-Datenblatt)
    porta.enable=0;
    delay_ms(15);  // 15 ms Pause
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);      // 3-mal 0011 mit mind. 5ms Pause
       delay_ms(5);
    }
    lcd_send_nibble(2);         // 0010 = 4-Bit Interface
    for(i=0;i<=3;++i)
       lcd_send_byte(0,LCD_INIT_STRING[i]);
}

void lcd_gotoxy( byte x, byte y) {
   byte address;

   if (y==1) address=0;
   if (y==2) address=0x40;
   if (y==3) address=0x10;
   if (y==4) address=0x50;
   address+=x-1;
   lcd_send_byte(0,0x80|address);
}

void lcd_putc( char c) {
 switch (c) {
     case '\f'   : lcd_send_byte(0,1);
                   delay_ms(2);
                                           break;
     case '\1'   : lcd_gotoxy(1,1);        break;
     case '\2'   : lcd_gotoxy(1,2);        break;
     case '\3'   : lcd_gotoxy(1,3);        break;
     case '\4'   : lcd_gotoxy(1,4);        break;
     default     : lcd_send_byte(1,c);     break;
 }
}


